'''
 *   Copyright (C) 2016 Alex Koren
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
'''

import serial
import time
import sys
import os
import re

e = b"\xff\xff\xff"
#device='/dev/ttyS0'   #Serial UART0
#device='/dev/ttyAMA0' #Serial UART1
device='/dev/ttyUSB0'   # Serial USB
file=b"GPS_clock.tft"

def getBaudrate(ser, fSize=None, checkModel=None):
    for baudrate in (38400, 115200, 9600, 2400, 4800, 19200, 38400, 57600, 115200):
        ser.baudrate = baudrate
        ser.timeout = 3000 / baudrate + .2
        print(('Trying with baudrate: ' + str(baudrate) + '...'))
        ser.write(b"\xff\xff\xff")
        ser.write(b'connect')
        ser.write(b"\xff\xff\xff")
        r = ser.read(128)
        rstr = ser.read(128).decode('utf-16')
        if b'comok' in r:
            print(('Connected with baudrate: ' + str(baudrate) + '...'))
            noConnect = False
            print(r)
            #status, unknown1, model, fwversion, mcucode, serial, flashSize = r.strip('\xff\x00').split(',')
            status = r[6:7].decode()
            model= r[16:26].decode()
            fwversion = r[32:34].decode()
            mcucode = r[35:40].decode() 
            serial = r[41:57].decode()
            flashSize = r[58:65].decode()
            flashMem = int(flashSize)
            if status == "1":
                print('Touchscreen: yes')
            else:
                print('Touchscreen: no')
            print(('Model: ' + model))
            print(('Firmware version: ' + fwversion))
            print(('MCU code: ' + mcucode))
            print(('Serial: ' + serial))
            print(('Flash size: ' + flashSize))
            if fSize and fSize > flashMem:
                print('File too big!')
                return False
            if checkModel and not checkModel in model:
                print('Wrong Display!')
                return False
            print()
            print()
            return True
        else:
            print('Display not responding - reboot')
    return False

def setDownloadBaudrate(ser, fSize, baudrate):
    ser.write(b"")
    print('Writing *******')
    bob = ("whmi-wri " + str(fSize) + "," + str(baudrate) + ",0").encode()
    #print(bob)
    ser.write(bob)
    ser.write(b"\xff\xff\xff")
    time.sleep(.05)
    ser.baudrate = baudrate
    ser.timeout = .5
    #print("ready to read")
    print()
    r = ser.read(1)
    if b"\x05" in r:
        return True
    return False

def transferFile(ser, filename, fSize):
    with open(filename, 'rb') as hmif:
        dcount = 0
        while True:
            data = hmif.read(4096)
            if len(data) == 0:
                break
            dcount += len(data)
            ser.timeout = 5
            ser.write(data)
            sys.stdout.write('\rDownloading, %3.1f%%...'% (dcount / float(fSize) * 100.0))
            sys.stdout.flush()
            ser.timeout = .5
            time.sleep(0.5)
            r = ser.read(1)
            #print(dcount)
            if b"\x05" in r:
                continue
            else:
                #print("OUT")
                return False
                break
        print()
    return True

def upload(ser, filename, checkModel=None):
    if not getBaudrate(ser, os.path.getsize(filename), checkModel):
        print('Could not find baudrate')
        exit(1)

    if not setDownloadBaudrate(ser, os.path.getsize(filename), 115200):
        print('Could not set download baudrate')
        print()
        print('*****************************************************************')
        print('* Press nextion display to wake it up - then run tftwrite again *')
        print('*****************************************************************')
        exit(1)

    if not transferFile(ser, filename, os.path.getsize(filename)):
        print('Could not transfer file')
        exit(1)

    print('File transferred successfully')
    exit(0)

if __name__ == "__main__":
    if len(sys.argv) != 4 and len(sys.argv) != 3:
        print('usage:\npython nextion.py file_to_upload.tft /path/to/dev/ttyDevice [nextion_model_name]\
        \nexample: nextion.py newUI.tft /dev/ttyUSB0 NX3224T024\
        \nnote: model name is optional')
        ser = serial.Serial(device, 38400, timeout=5)
        upload(ser, file, None)
        exit(1)

    try:
        ser = serial.Serial(sys.argv[2], 38400, timeout=5)
    except serial.serialutil.SerialException:
        print(('could not open serial device ' + sys.argv[2]))
        exit(1)
    if serial.VERSION <= "3.0":
        if not ser.isOpen():
            ser.open()
    else:
        if not ser.is_open:
            ser.open()

    checkModel = None
    if len(sys.argv) == 4:
        checkModel = sys.argv[3]
        pattern = re.compile("^NX\d{4}[TFK]\d{3}$")
        if not pattern.match(checkModel):
            print('Invalid model name. Please give a correct one (e.g. NX3224T024)')
            exit(1)
    upload(ser, sys.argv[1], checkModel)



# listSats by Andrew BARRON
import serial                    # Py serial use pip3 to install
import serial.tools.list_ports   # Py serial tools use pip3 to install - used to find current USB port
import numpy as np               # Used for radar plot polar to cartesian calcs  
from time import sleep           # used to catch up on rest after a hard day

vGNS = "GSV"
vGNS1 = "$GLGSV"
vGNS2 = "$GPGSV"
z=0
tlist=["","","","","","","","","","","","","",""]

vmark = 0
xmark = 0
origonx = 129
origony = 122
colour = "RED"
terminator = b'\xff\xff\xff'  # used to terminate every command
quote=b"\x22"                 # used to encase strings in quotes
device='/dev/ttyUSB0'         # this is the Nextion screen device - the program will track changes in the USB number
device2='/dev/ttyS0'          # this is the GPS device. If it says dev/ttyAMA0 the serial USB device is probably not working
Terminal ="YES"               # Set this to "YES" to turn on the terminal output. Anything else is Nextion output only.  

class Nextion:
   def __init__(self):
        self.ser = serial.Serial(device, 38400, timeout=1)

   def connect(self):
       self.ser.write(b"connect" + terminator)
       r = self.ser.read(128)
       return r

   def coms(self,command):
       self.ser.write(command.encode() + terminator)
       r = self.ser.readline()
       #print(r)
       if b'p' in r:  # text
          vtxt= str(r[1:12])
          return vtxt
       #if b'q' in r:  # a number
       #  vnum=int.from_bytes(r[1:2],"little")
       #   return vnum
       return 0

   def set(self,command, var):
       self.ser.write(command.encode() + quote + var.encode() +quote + terminator)

##################################
class GPS:
   def __init__(self):
        self.gps = serial.Serial(device2, 9600, timeout=1)

   def getGPS(self):
       raw = self.gps.readline()
       return raw
       

##########################################
def main():
  global xmark
  n.coms("pic 0,0,4")
  n.coms("page 5")
  while True:
    data = g.getGPS()
    # print(data)  # this is a bytearray type
    if data is not None:
        # convert bytearray to string
        data_string = "".join([chr(b) for b in data])
        #print(data_string, end="")
        vMessage=  data_string[3:6]
        if vMessage==vGNS:     
           #print(data_string, end="")
           part = data_string.split(",")
           vLen=len(part)
           vSats=part[3]
           vID=part[4]
           vCon=int(vID)
           dispID(vCon,vID)
           if vCon<33:
              vType=" GPS sat ID "
              vmark=1 
              n.set("page5.t1.txt=",vSats)
           elif vCon>32 and vCon<65:
              vType=" SBAS sat ID "
              print("SBAS")
              vmark=1 
           elif vCon>64 and vCon<97:
              vType=" Glonass sat ID "
              vmark=2 
              n.set("page5.t0.txt=",vSats)
           else:
              vType=" satellite ID "
              vmark=3 
           if not xmark==vmark:
              if Terminal =="YES":
                 print("*************************************")
           xmark=vmark
           vEL=part[5]
           vAZ=part[6]
           vRSSI=part[7]
           if vRSSI=="":
              vRSSI="*"   
           if '*' in vRSSI: 
              vOut=vSats +vType + vID +" Elevation "+ vEL +" Azimuth "+ vAZ  
           else: 
              vOut=vSats +vType + vID +" Elevation "+ vEL +" Azimuth "+ vAZ +" Signal "+ vRSSI   
           if Terminal =="YES":
              print(vOut)
           if vmark==1:
              display(vAZ,vEL,"RED") 
           elif vmark==2:
              display(vAZ,vEL,"GREEN") 
           else:
              display(vAZ,vEL,"YELLOW") 
           if vLen>8:  
              vID=part[8]
              vCon=int(vID)
              dispID(vCon,vID)
              if vCon>32 and vCon<65:
                 vType=" SBAS sat ID "
              vEL=part[9]
              vAZ=part[10]
              vRSSI=part[11]
              if vRSSI=="":
                 vRSSI="*"   
              if '*' in vRSSI: 
                 vOut=vSats +vType + vID +" Elevation "+ vEL +" Azimuth "+ vAZ  
              else: 
                 vOut=vSats +vType + vID +" Elevation "+ vEL +" Azimuth "+ vAZ +" Signal "+ vRSSI   
              if Terminal =="YES":
                 print(vOut)
              if vmark==1:
                 display(vAZ,vEL,"RED") 
              elif vmark==2:
                 display(vAZ,vEL,"GREEN") 
              else:
                 display(vAZ,vEL,"YELLOW") 
           if vLen>12:  
              vID=part[12]
              vCon=int(vID)
              dispID(vCon,vID)
              if vCon>32 and vCon<65:
                 vType=" SBAS sat ID "
              vEL=part[13]
              vAZ=part[14]
              vRSSI=part[15]
              if vRSSI=="":
                 vRSSI="*"   
              if '*' in vRSSI: 
                vOut=vSats +vType + vID +" Elevation "+ vEL +" Azimuth "+ vAZ  
              else: 
                 vOut=vSats +vType + vID +" Elevation "+ vEL +" Azimuth "+ vAZ +" Signal "+ vRSSI   
              if Terminal =="YES":
                 print(vOut)
              if vmark==1:
                 display(vAZ,vEL,"RED") 
              elif vmark==2:
                 display(vAZ,vEL,"GREEN") 
              else:
                 display(vAZ,vEL,"YELLOW") 
           if vLen>16:  
              vID=part[12]
              vCon=int(vID)
              dispID(vCon,vID)
              if vCon>32 and vCon<65:
                 vType=" SBAS sat ID "
              vEL=part[13]
              vAZ=part[14]
              vRSSI=part[15]
              if vRSSI=="":
                 vRSSI="*"   
              if '*' in vRSSI: 
                vOut=vSats +vType + vID +" Elevation "+ vEL +" Azimuth "+ vAZ  
              else: 
                 vOut=vSats +vType + vID +" Elevation "+ vEL +" Azimuth "+ vAZ +" Signal "+ vRSSI   
              if Terminal =="YES":
                 print(vOut)
              if vmark==1:
                 display(vAZ,vEL,"RED") 
              elif vmark==2:
                 display(vAZ,vEL,"GREEN") 
              else:
                 display(vAZ,vEL,"YELLOW") 

########################################## 
def display(AZ,EL,colour):
    global z
    az=float(AZ)-90
    el=(90-float(EL))*1.05
    #print("AZ "+str(az)+" EL "+str(el))
    rad= np.deg2rad(az)
    x = float(el) * np.cos(rad)
    y = float(el) * np.sin(rad)
    x+= origonx 
    y += origony
    xs=int(x)
    ys=int(y)
    comm="cirs "+str(xs)+","+str(ys)+",3,"+colour
    n.coms(comm)
    #print(comm)
    z+=1
    #if z==50:
    #   z=0
    #   n.coms("pic 0,0,4")
##########################################
def dispID(vCon,vID):
    global tlist
    if vCon<65:
       tlist[13]=tlist[11]
       tlist[11]=tlist[9]
       tlist[9]=tlist[7]
       tlist[7]=tlist[5]
       tlist[5]=tlist[3]
       tlist[3]=vID
       n.set("page5.t3.txt=",tlist[3])  
       n.set("page5.t5.txt=",tlist[5])  
       n.set("page5.t7.txt=",tlist[7])  
       n.set("page5.t9.txt=",tlist[9])  
       n.set("page5.t11.txt=",tlist[11])  
       n.set("page5.t13.txt=",tlist[13])  
    if vCon>64:
       tlist[12]=tlist[10]
       tlist[10]=tlist[8]
       tlist[8]=tlist[6]
       tlist[6]=tlist[4]
       tlist[4]=tlist[2]
       tlist[2]=vID
       n.set("page5.t2.txt=",tlist[2])  
       n.set("page5.t4.txt=",tlist[4])  
       n.set("page5.t6.txt=",tlist[6])  
       n.set("page5.t8.txt=",tlist[8])  
       n.set("page5.t10.txt=",tlist[10])  
       n.set("page5.t12.txt=",tlist[12]) 


####################### this finds the acive USB port numer which will increment every time the USB dongle is plugged in. It resets to USB0 on reboot
def checkUSB():
   global device
   myports = [tuple(p) for p in list(serial.tools.list_ports.comports())]
   #print (myports)
   bob=str(myports[0:1])
   device=(bob[3:15])
   print(device)

checkUSB()
n=Nextion()
g=GPS()
main()





# * This program displays the raw data packets from an Adafruit GPS breaout board connected to RPi serial port 0 
# No satellite fix the LED blinks evey 2 seconds.
# After sat fix the LED blinks every 10 seconds.
# Test serial stream. Should list NMEA strings until stopped with CTRL-C 

import os
os.system('stty -F /dev/serial0 raw 9600 cs8 clocal -cstopb')
# sets serial port 0 to 9600 bauds, with 8 bits, no modem control, and 1 stop bit
os.system('cat /dev/serial0')
# displays the NMEA strings as raw data





# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

# Simple GPS module demonstration.
# Will print NMEA sentences received from the GPS, great for testing connection
# Uses the GPS to send some commands, then reads directly from the GPS
import time
import board
#import busio

import adafruit_gps

# Create a serial connection for the GPS connection using default speed and
# a slightly higher timeout (GPS modules typically update once a second).
# These are the defaults you should use for the GPS FeatherWing.
# For other boards set RX = GPS module TX, and TX = GPS module RX pins.
# uart = busio.UART(board.TX, board.RX, baudrate=9600, timeout=10)

# for a computer, use the pyserial library for uart access
import serial
uart = serial.Serial("/dev/ttyS0", baudrate=9600, timeout=10)

# If using I2C, we'll create an I2C interface to talk to using default pins
# i2c = board.I2C()

# Create a GPS module instance.
gps = adafruit_gps.GPS(uart)  # Use UART/pyserial
# gps = adafruit_gps.GPS_GtopI2C(i2c)  # Use I2C interface

# Initialize the GPS module by changing what data it sends and at what rate.
# These are NMEA extensions for PMTK_314_SET_NMEA_OUTPUT and
# PMTK_220_SET_NMEA_UPDATERATE but you can send anything from here to adjust
# the GPS module behavior:
#   https://cdn-shop.adafruit.com/datasheets/PMTK_A11.pdf

# Turn on the basic GGA and RMC info (what you typically want)
gps.send_command(b"PMTK314,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0")
# Turn on just minimum info (RMC only, location):
# gps.send_command(b'PMTK314,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0')
# Turn off everything:
# gps.send_command(b'PMTK314,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0')
# Tuen on everything (not all of it is parsed!)
# gps.send_command(b'PMTK314,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0')

# Set update rate to once a second (1hz) which is what you typically want.
gps.send_command(b"PMTK220,1000")
# Or decrease to once every two seconds by doubling the millisecond value.
# Be sure to also increase your UART timeout above!
# gps.send_command(b'PMTK220,2000')
# You can also speed up the rate, but don't go too fast or else you can lose
# data during parsing.  This would be twice a second (2hz, 500ms delay):
# gps.send_command(b'PMTK220,500')

# Main loop runs forever printing data as it comes in
timestamp = time.monotonic()
while True:
    data = gps.read(32)  # read up to 32 bytes
    # print(data)  # this is a bytearray type

    if data is not None:
        # convert bytearray to string
        data_string = "".join([chr(b) for b in data])
        print(data_string, end="")

    if time.monotonic() - timestamp > 5:
        # every 5 seconds...
        gps.send_command(b"PMTK605")  # request firmware version
        timestamp = time.monotonic()



#! /usr/bin/python
# Written by Dan Mandle http://dan.mandle.me September 2012
# License: GPL 2.0
 
import os
from gps import *
from time import *
import time
import threading
 
gpsd = None #seting the global variable
count = 0
 
os.system('clear') #clear the terminal (optional)
 
class GpsPoller(threading.Thread):
  def __init__(self):
    threading.Thread.__init__(self)
    global gpsd #bring it in scope
    gpsd = gps(mode=WATCH_ENABLE) #starting the stream of info
    self.current_value = None
    self.running = True #setting the thread running to true
 
  def run(self):
    global gpsd
    while gpsp.running:
      gpsd.next() #this will continue to loop and grab EACH set of gpsd info to clear the buffer

if __name__ == '__main__':
  gpsp = GpsPoller() # create the thread
  try:
    gpsp.start() # start it up
    while True:
      #It may take a second or two to get good data
      print (gpsd.fix.latitude,', ',gpsd.fix.longitude,'  Time: ',gpsd.utc)
 
      os.system('clear')
      count = count+1
      print ()
      print (' GPS reading')
      print ('----------------------------------------')
      print ('latitude    ' , gpsd.fix.latitude)
      print ('longitude   ' , gpsd.fix.longitude)
      print ('time utc    ' , gpsd.utc,' + ', gpsd.fix.time)
      print ('altitude (m)' , gpsd.fix.altitude)
      print ('eps         ' , gpsd.fix.eps)
      print ('epx         ' , gpsd.fix.epx)
      print ('epv         ' , gpsd.fix.epv)
      print ('ept         ' , gpsd.fix.ept)
      print ('speed (m/s) ' , gpsd.fix.speed)
      print ('climb       ' , gpsd.fix.climb)
      print ('track       ' , gpsd.fix.track)
      print ('mode        ' , gpsd.fix.mode)
      print
      print ('sats        ' , gpsd.satellites)
      print ('Loop count  ' , count) 
 
      time.sleep(3) #set to whatever
 
  except (KeyboardInterrupt, SystemExit): #when you press ctrl+c
    print ("\nKilling Thread...")
    gpsp.running = False
    gpsp.join() # wait for the thread to finish what it's doing
  print ("Done.\nExiting.")




# dispMap by Andrew BARRON
# this will display your position on a map preloaded onto the Nextion screen. You need to enter the top left and bottom right lat and lon 
# into the olat, olon, mlat and mlon variables

import serial                    # Py serial use pip3 to install
import serial.tools.list_ports   # Py serial tools use pip3 to install - used to find current USB port
import staticmaps                # use pip3 install py-staticmaps  to install - used to find a map of your location online RPI must have WiFi Internet access

Get_Map = "NO"                   # set to "YES" to download a map based on your coordinates. Note that this must be manually added to the Nextion screen using the Nextion editor.


mapx=310  # Enter the map size in pixels from the Nextion Editor
mapy=240

olat=83  # Enter the top left and bottom right coordinates
olon=164
mlat=-61
mlon=22
colour = "RED"
terminator = b'\xff\xff\xff'  # used to terminate every command
quote=b"\x22"                 # used to encase strings in quotes
device='/dev/ttyUSB0'         # this is the Nextion screen device - the program will track changes in the USB number
device2='/dev/ttyS0'          # this is the GPS device. Might be dev/ttyAMA0 or /dev/serial0


class Nextion:
   def __init__(self):
        self.ser = serial.Serial(device, 38400, timeout=1)

   def connect(self):
       self.ser.write(b"connect" + terminator)
       r = self.ser.read(128)
       return r

   def coms(self,command):
       self.ser.write(command.encode() + terminator)
       r = self.ser.readline()
       #print(r)
       if b'p' in r:  # text
          vtxt= str(r[1:12])
          return vtxt
       #if b'q' in r:  # a number
       #  vnum=int.from_bytes(r[1:2],"little")
       #   return vnum
       return 0

   def set(self,command, var):
       self.ser.write(command.encode() + quote + var.encode() +quote + terminator)

##################################
class GPS:
   def __init__(self):
        self.gps = serial.Serial(device2, 9600, timeout=1)

   def getGPS(self):
       raw = self.gps.readline()
       return raw
       

##########################################
def main():
  global xmark, Get_Map
  n.coms("page 6")
  while True:
    data = g.getGPS()   # this is a bytearray type
    if data is not None:
        # convert bytearray to string
        data_string = "".join([chr(b) for b in data])
        vMessage=  data_string[0:6]
        if vMessage=="$GNRMC" or  vMessage=="$GPRMC":     # Find a "$GPRMC" NMEA string
           #print(data_string, end="")
           parts = data_string.split(",")
           vlatf = formatDegreesMinutes(parts[3], 2)
           if parts[4]=="S":
              vlatf=-vlatf
           vLat = str(round(vlatf,3))
           vlonf = formatDegreesMinutes(parts[5], 3)
           if parts[6]=="W":
              vlonf=-vlonf
           vLon = str(round(vlonf,3))
           if Get_Map == "YES":
              print("get a map")
              getMap(vlatf,vlonf)
              Get_Map = "NO"
           #print ("Your position: lon = " + vLon + ", lat = " + vLat)
           n.set("page6.t20.txt=",vLat) # print Lat
           n.set("page6.t21.txt=",vLon) # print Lon
           plot(vlatf,vlonf,colour)


           vlatf=51.50377  # London
           vlonf=-0.11740  # London
           plot(vlatf,vlonf,colour)
         
           vlatf=25.85850 # Miami
           vlonf=-80.19426  
           plot(vlatf,vlonf,colour)

           vlatf=34.03235 # LA
           vlonf=-118.25994  
           plot(vlatf,vlonf,colour)


#######################
def formatDegreesMinutes(coordinates, digits):
              # In the NMEA message, the position gets transmitted as: DDMM.MMMMM, where DD denotes the degrees and MM.MMMMM denotes
              # the minutes. However, I want to convert this format to the following: DD.MMMM. This method converts a transmitted string to the desired format
    #print(coordinates)
    #print(digits)

    parts = coordinates.split(".")
    if (len(parts) != 2):
        return coordinates

    if (digits > 3 or digits < 2):
        return coordinates
    
    degs = coordinates[:digits]
    y=int(degs)
    mins= (coordinates[digits:digits+7])
    x = round(float(mins)/60,6)
    degrees=(y+x)
    #print(degrees)
    return degrees
########################################## 
def plot(lat,lon,colour):
    global mapx,mapy,olat,olon,mlat,mlon
    #print(lat),print(lon)
    maplat=((lat-olat)/(mlat-olat))*mapy
    maplon=((lon-olon)/(mlon-olon))*mapx
    pixels=mapx/(360+mlon-olon)
    if lon>olon:
       maplon=(lon-olon)*pixels
    elif lon<0:
       maplon=(360-olon+lon)*pixels
    xs=int(maplon)
    ys=int(maplat)
    #print(maplat,maplon)
    #print(xs,ys)

    comm="cir "+str(xs)+","+str(ys)+",3,"+colour
    n.coms(comm)
    comm="cirs "+str(xs)+","+str(ys)+",2,"+"YELLOW"
    n.coms(comm)
    comm="cirs "+str(xs)+","+str(ys)+",1,"+colour
    n.coms(comm)


####################### this finds the acive USB port numer which will increment every time the USB dongle is plugged in. It resets to USB0 on reboot
def checkUSB():
   global device
   myports = [tuple(p) for p in list(serial.tools.list_ports.comports())]
   #print (myports)
   bob=str(myports[0:1])
   device=(bob[3:15])
   print(device)

########################
def getMap(vlatf,vlonf):
   context = staticmaps.Context()
   #context.set_tile_provider(staticmaps.tile_provider_StamenToner)        # Alternate map design (black)
   #context.set_tile_provider(staticmaps.tile_provider_ArcGISWorldImagery) # Alternate map design (photo)
   context.set_tile_provider(staticmaps.tile_provider_OSM)                 # Alternate map design (normal)

   center1 = staticmaps.create_latlng(vlatf,vlonf)
   zoom = 5000  # radius in km 1-5000km

   context.add_object(staticmaps.Circle(center1, zoom, fill_color=staticmaps.TRANSPARENT, color=staticmaps.WHITE, width=1))
   #context.add_object(staticmaps.Marker(center1, color=staticmaps.RED, size=6))
   print("This will take up to 60 seconds") 
   # render non-anti-aliased png
   image = context.render_pillow(320, 240)
   image.save("map.png")
   print("map.png saved") 
   print("Sadly you have to use the Nextion eitor to load it onto page 6")
   print("plotting")

checkUSB()
n=Nextion()
g=GPS()
main()





# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

# Simple GPS module demonstration.
# Will wait for a fix and print a message every second with the current location
# and other details.
import time
import board
#import busio
import adafruit_gps


# Create a serial connection for the GPS connection using default speed and
# a slightly higher timeout (GPS modules typically update once a second).
# These are the defaults you should use for the GPS FeatherWing.
# For other boards set RX = GPS module TX, and TX = GPS module RX pins.
#uart = busio.UART(board.TX, board.RX, baudrate=9600, timeout=10)

# for a computer, use the pyserial library for uart access
import serial
uart = serial.Serial("/dev/ttyS0", baudrate=9600, timeout=10)


# If using I2C, we'll create an I2C interface to talk to using default pins
# i2c = board.I2C()

# Create a GPS module instance.
gps = adafruit_gps.GPS(uart, debug=False)  # Use UART/pyserial
# gps = adafruit_gps.GPS_GtopI2C(i2c, debug=False)  # Use I2C interface

# Initialize the GPS module by changing what data it sends and at what rate.
# These are NMEA extensions for PMTK_314_SET_NMEA_OUTPUT and
# PMTK_220_SET_NMEA_UPDATERATE but you can send anything from here to adjust
# the GPS module behavior:
#   https://cdn-shop.adafruit.com/datasheets/PMTK_A11.pdf

# Turn on the basic GGA and RMC info (what you typically want)
gps.send_command(b"PMTK314,0,1,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0")
# Turn on just minimum info (RMC only, location):
# gps.send_command(b'PMTK314,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0')
# Turn off everything:
# gps.send_command(b'PMTK314,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0')
# Turn on everything (not all of it is parsed!)
# gps.send_command(b'PMTK314,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0')

# Set update rate to once a second (1hz) which is what you typically want.
gps.send_command(b"PMTK220,1000")
# Or decrease to once every two seconds by doubling the millisecond value.
# Be sure to also increase your UART timeout above!
# gps.send_command(b'PMTK220,2000')
# You can also speed up the rate, but don't go too fast or else you can lose
# data during parsing.  This would be twice a second (2hz, 500ms delay):
# gps.send_command(b'PMTK220,500')

# Main loop runs forever printing the location, etc. every second.
last_print = time.monotonic()
while True:
    # Make sure to call gps.update() every loop iteration and at least twice
    # as fast as data comes from the GPS unit (usually every second).
    # This returns a bool that's true if it parsed new data (you can ignore it
    # though if you don't care and instead look at the has_fix property).
    gps.update()
    # Every second print out current location details if there's a fix.
    current = time.monotonic()
    if current - last_print >= 1.0:
        last_print = current
        if not gps.has_fix:
            # Try again if we don't have a fix yet.
            print("Waiting for fix...")
            continue
        # We have a fix! (gps.has_fix is true)
        # Print out details about the fix like location, date, etc.
        print("=" * 40)  # Print a separator line.
        print(
            "Fix timestamp: {}/{}/{} {:02}:{:02}:{:02}".format(
                gps.timestamp_utc.tm_mon,  # Grab parts of the time from the
                gps.timestamp_utc.tm_mday,  # struct_time object that holds
                gps.timestamp_utc.tm_year,  # the fix time.  Note you might
                gps.timestamp_utc.tm_hour,  # not get all data like year, day,
                gps.timestamp_utc.tm_min,  # month!
                gps.timestamp_utc.tm_sec,
            )
        )
        print("Latitude: {0:.6f} degrees".format(gps.latitude))
        print("Longitude: {0:.6f} degrees".format(gps.longitude))
        print("Fix quality: {}".format(gps.fix_quality))
        # Some attributes beyond latitude, longitude and timestamp are optional
        # and might not be present.  Check if they're None before trying to use!
        if gps.satellites is not None:
            print("# satellites: {}".format(gps.satellites))
        if gps.altitude_m is not None:
            print("Altitude: {} meters".format(gps.altitude_m))
        if gps.speed_knots is not None:
            print("Speed: {} knots".format(gps.speed_knots))
        if gps.track_angle_deg is not None:
            print("Track angle: {} degrees".format(gps.track_angle_deg))
#        if gps.horizontal_dilution is not None:
#            print("Horizontal dilution: {}".format(gps.horizontal_dilution))
        if gps.height_geoid is not None:
            print("Height geoid: {} meters".format(gps.height_geoid))



'''
 *   Copyright (C) 2022 Andrew Barron
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
'''
import serial                    # Py serial use pip3 to install
import serial.tools.list_ports   # Py serial tools use pip3 to install
import time			 # time used for delays
from threading import Thread     # used to create program threads

# sets the initial values for global variables. Stops errors caused by variables being called before they are set in another thread.
terminator = b'\xff\xff\xff'  # used to terminate every command
quote=b"\x22"                 # used to encase strings in quotes
vDatef ="0"                   # initial value dd/mm/yy or mm/dd/yy format
vTimef="0"                    # initial value 24 or 12 hour time format
vGridf="0"                    # initial value 6 or 8 digit grid
vDate=""	              # initial value for the date	
Uoff=13	                      # initial value for UTC offset
vLat="42.497 S"               # initial value for Latitude
vLon="72.605 E"               # initial value for Longitude
vGrid=""                      # initial value for Maidenhead grid
vSpeed="0"                    # initial value for speed
vBearing="0"                  # initial value for bearing
vAlt="0"                      # initial value for altitude
vData=""                      # initial value for data recieved from the GPS module 
aData=""                      # initial value for $GPGGA data string from getGPS() to dispGPS()
pData=""                      # initial value for $GPRMC data string from getGPS() to dispGPS()
vNMEA="no data received"      # initial value for Information string
vDadd=0                       # used to add or subtract a day to the date if local date is different to UTC date from GPS
page=0                        # initial value for Nextion page
LCLtime="00:00:00"            # initial value for local time
UTCtime="00:00:00"            # initial value for UTC time
device='/dev/ttyUSB0'         # this is the Nextion screen device - the program will track changes in the USB number
device2='/dev/ttyS0'          # this is the GPS device. Might be dev/ttyAMA0 or /dev/serial0

##################################
class Nextion:
   def __init__(self):
        checkUSB()
        self.ser = serial.Serial(device, 38400, timeout=1)

   def connect(self):
       self.ser.write(b"connect" + terminator)
       r = self.ser.read(128)
       return r

   def get(self,command):
       self.ser.write(command.encode() + terminator)
       r = self.ser.readline()
       #print(r)
       if b'p' in r:  # text
          vtxt= str(r[1:12])
          return vtxt
       #if b'q' in r:  # a number
       #  vnum=int.from_bytes(r[1:2],"little")
       #   return vnum
       return 0

   def set(self,command, var):
       self.ser.write(command.encode() + quote + var.encode() +quote + terminator)

##################################
class GPS:
   def __init__(self):
        self.gps = serial.Serial(device2, 9600, timeout=1)

   def getGPS(self):
       raw = self.gps.readline()
       return raw
       

####################### this finds the acive USB port numer which will increment every time the USB dongle is plugged in. It resets to USB0 on reboot
def checkUSB():
   global device
   myports = [tuple(p) for p in list(serial.tools.list_ports.comports())]
   #print (myports)
   bob=str(myports[0:1])
   device=(bob[3:15])
   print(device)
####################### reads the Nextion buttons and UTC offset from the display. Takes about 1 second.
def getData():
   global Uoff, page, vDatef, vTimef, vGridf
   vt=page
   while True:
      data=n.get("get page4.vstring.txt")
      if not '\xff' in data: 
         #print(data)   
         part = data.split(",")
         vdatef=part[0]
         vDatef=vdatef[2]
         vTimef=part[1]
         vGridf=part[2]
         Uoff=int(part[3])
         page=int(part[4])
         Uoff-=13
         #print("UTC offset " + str(Uoff))
         #print(vDatef+" "+vTimef+" "+vGridf) 
#######################
def setData():
   while True:
      if page==0 or page==1:
        n.set("page0.lTime.txt=",LCLtime)
        n.set("page0.uTime.txt=",UTCtime)
        n.set("page1.lTime.txt=",LCLtime)
        n.set("page1.uTime.txt=",UTCtime)
        n.set("page1.lDate.txt=",vDate)
        time.sleep(.5)
      if page==2:
        #print("setData page " +str(page))
        #print("setData grid " +vGrid)
        n.set("page2.Maiden.txt=",vGrid)
        n.set("page2.t20.txt=",vLat)
        n.set("page2.t21.txt=",vLon)
        time.sleep(1)
      if page==3:
        #print("setData page " +str(page))
        n.set("page3.NMEA.txt=",vNMEA)
        n.set("page3.tsat.txt=",vSats)
        n.set("page3.tspeed.txt=",vSpeed)
        n.set("page3.tnth.txt=",vBearing)
        n.set("page3.talt.txt=",vAlt)
        time.sleep(1)
      if page==4:
        #print("setData page " +str(page))
        time.sleep(1)
        pass

#######################
def initialise():
    global page
    r=n.connect()
    #print(r)
    if b'comok' in r:
            print('Connected')
            noConnect = False
            ## string format = status, unknown1, model, fwversion, mcucode, serial, flashSize = r.strip('\xff\x00').split(',')
            status = r[6:7].decode()
            model= r[16:31].decode()
            fwversion = r[32:34].decode()
            mcucode = r[35:40].decode() 
            serial = r[41:57].decode()
            flashSize = r[58:65].decode()
            flashMem = int(flashSize)
            if status == "1":
                print('Touchscreen: yes')
            else:
                print('Touchscreen: no')
            print(('Model: ' + model))
            print(('Firmware version: ' + fwversion))
            print(('MCU code: ' + mcucode))
            print(('Serial: ' + serial))
            print(('Flash size: ' + flashSize))
            n.set("page0.t2.txt=","UTC") # set text on page 0
            n.get("sleep=0")  # wake up display
            page=n.get("sendme")
    else:
            print('Display not responding - reboot')
    print("Initialised")
    print()

#######################
def getGPS():
    print("get GPS")
    global UTCtime, LCLtime, aData, pData, page, vDadd
    while True:
      mbytes = g.getGPS()
      if len(mbytes)>0: 
          vData = mbytes.decode()  
          vMessage = vData[0:6]
          #print(vMessage)
          if (vMessage == "$GPRMC" or vMessage == "$GNRMC"):   # Get the position data that was transmitted with the GPRMC message
             pData=vData 
             #print(vMessage)
          if (vMessage == "$GPGGA" or vMessage == "$GNGGA"):   # Get the position data that was transmitted with the GPGGA message
             aData=vData
             if page==0 or page==1:    # format UTC and find Local Time
                parts = aData.split(",")
                utime=parts[1]
                UTCtime= utime[0:2]+":"+utime[2:4]+":"+utime[4:6] # format UTC time
                hrs=int(utime[0:2])+Uoff  # apply the UTC offset to find the Local time
                vDadd =0               
                if hrs < 0:               # reset if the UTC offset create a time less than 0
                   hrs = hrs + 24
                   vDadd =-1              # local date different to UTC date
                if hrs > 24:              # reset if the UTC offset create a time over 24 hours
                   hrs = hrs - 24
                   vDadd =1               # local date different to UTC date
                if vTimef =="1":          # If 12 hour clock set AM or PM
                   if hrs > 12:
                      hrs=hrs-12
                      shrs = str(hrs)
                      LCLtime = shrs +":"+utime[2:4]+" PM" 
                   else:
                      shrs = str(hrs)
                      LCLtime = shrs +":"+utime[2:4]+" AM" 
                else:
                   shrs = str(hrs)
                   LCLtime = shrs +":"+utime[2:4]+":"+utime[4:6] # format 24 hour Local time
             time.sleep(0.5)
      else:
          vSats="00"
####################### dispGPS() displays the date, grid, lat, lon, NMEA text, vDadd
def dispGPS(): 
      print("disp GPS")
      global vDate, vLat, vLon, vNMEA, vSats, vSpeed, vBearing, vAlt
      #print(aData)
      #print(pData)
      vDate="0"
      vspeed="0.00"
      vSats="00"
      vAlt="0"
      vbearing="0.00"
      speed=1.234
      bearing=1.234 #???
      lst = [0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00]
      lstb = [0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00]
      while True: 
         vRec1=0
         vRec2=0
         # do GPS magic
         vMessage = aData[0:6]
         #print(vMessage)
         if (vMessage == "$GPGGA" or vMessage == "$GNGGA"):   # Get the position data that was transmitted with the GPGGA message
            parts = aData.split(",")
            vSats= parts[7]
            if(vSats!="00"):
              vRec1=1  
            vlonf = formatDegreesMinutes(parts[4], 3)
            vlonr = round(vlonf,3)
            vLon=str(vlonr)+" "+parts[5]
            vlatf = formatDegreesMinutes(parts[2], 2)
            vlatr = round(vlatf,3)
            vLat=str(vlatr)+" "+parts[3]
            #print ("Your position: lon = " + str(vLon) + ", lat = " + str(vLat))
            vAlt=parts[9]
            if page==2:
               calcGrid(vlonf,vlatf, parts[5],parts[3])  ## calculate the grid

         vMessage = pData[0:6]
         if (vMessage == "$GPRMC" or vMessage == "$GNRMC"):   # Get the position data that was transmitted with the GPRMC message
             parts = pData.split(",")
             #print(pData)
             valid=parts[2]
             if(valid=="A"):
                vRec2=1  
             vspeed=parts[7]
             vbearing=parts[8]
             if vbearing=="":
                vbearing=0 
             vdate=parts[9]
             dd=vdate[0:2]
             mm=vdate[2:4]
             yy=vdate[4:6]
             d=int(dd)+vDadd
             m=int(mm)
             if d>31:
                d=1
                m=m+1
             if d<1:
                d=31
                m=m-1
             dd=str(d)
             mm=str(m)  
             if d<10:
                dd="0"+str(d)
             if m<10:
                mm="0"+str(m)

             if vDatef =="1":
                vDate=mm+"/"+dd+"/"+yy
             else:  
               vDate=dd+"/"+mm+"/"+yy
             #print("date "+vDate) 
             # convert speed to kph and mph
             lst[9]=lst[8]
             lst[8]=lst[7]
             lst[7]=lst[6]
             lst[6]=lst[5]
             lst[5]=lst[4]
             lst[4]=lst[3]
             lst[3]=lst[2]
             lst[2]=lst[1]
             lst[1]=lst[0]
             lst[0]=float(vspeed)
             speed = Average(lst)
             kph=round(speed*1.852,1)
             mph=round(speed*1.15078,1)
             vSpeed=(str(kph)+" kph  "+str(mph)+" mph")

             # average bearing
             lstb[9]=lstb[8]
             lstb[8]=lstb[7]
             lstb[7]=lstb[6]
             lstb[6]=lstb[5]
             lstb[5]=lstb[4]
             lstb[4]=lstb[3]
             lstb[3]=lstb[2]
             lstb[2]=lstb[1]
             lstb[1]=lstb[0]
             lstb[0]=float(vbearing)
             vBearing = str(round(Average(lstb),1))
             #print("Bearing "+str(vbearing)+" averaged "+str(vBearing))
             time.sleep(0.5)
         if(vRec1==1 and vRec2==1):
            vNMEA="GGA and RMC received"
         elif(vRec1==1):
            vNMEA="GGA received"
         elif(vRec2==1):
            vNMEA="RMC received"
         else: 
            vNMEA="Waiting for GPS lock"  
#######################
def formatDegreesMinutes(coordinates, digits):
              # In the NMEA message, the position gets transmitted as: DDMM.MMMMM, where DD denotes the degrees and MM.MMMMM denotes
              # the minutes. However, I want to convert this format to the following: DD.MMMM. This method converts a transmitted string to the desired format
    #print(coordinates)
    #print(digits)

    parts = coordinates.split(".")
    if (len(parts) != 2):
        #print("no coordinates")
        return 0

    if (digits > 3 or digits < 2):
        #print("coordinates length error")
        return 0
    
    degs = coordinates[:digits]
    y=int(degs)
    mins= (coordinates[digits:digits+7])
    x = round(float(mins)/60,6)
    degrees=(y+x)
    #print(degrees)
    return degrees
####################### Python program to get average of a list
def Average(lst):
    return sum(lst) / len(lst)
  
#######################
def calcGrid(vlon,vlat,ew, ns):
    global vGrid, vGridf
    if(ew=="W"):
      vlon=-vlon
    if(ns=="S"):
      vlat=-vlat
    #vlat=42.664048 # test FN42ep09
    #vlon=-71.661962 # test
    vlon1=vlon+180
    vlon2=vlon1/20
    out1=int(vlon2)
    rem1=vlon1-(out1*20)
    vlon3=rem1/2
    out3=int(vlon3)
    rem2=rem1-(out3*2)
    vlon4=rem2/0.083333
    out5=int(vlon4)
    rem3=rem2-(out5*0.083333)
    vlon5=(rem3/0.0083333)
    out7=int(vlon5)

    vlat1=vlat+90
    vlat2=vlat1/10
    out2=int(vlat2)
    vlat3=vlat1-(out2*10)
    out4=int(vlat3)
    rem2=vlat3-out4
    vlat4=rem2/0.0416665
    out6=int(vlat4)
    rem3=rem2-(out6*0.0416665)
    vlat5=(rem3/0.00416665)
    out8=int(vlat5)
    if(vGridf=="1"):    # sort out 6 digit or 8 digit Maidenhead grid 
        vGrid=chr(out1+65)+chr(out2+65)+str(out3)+str(out4)+chr(out5+97)+chr(out6+97)+str(out7)+str(out8)
    else:
        vGrid=chr(out1+65)+chr(out2+65)+str(out3)+str(out4)+chr(out5+97)+chr(out6+97)
 
#######################
def main():
   initialise()
   time.sleep(1)
   ### Threads ###
   print("Starting Threads")
   t1=Thread(target=getData)
   t2=Thread(target=setData)
   t3=Thread(target=getGPS)
   t4=Thread(target=dispGPS)


   t1.start()
   t2.start()
   t3.start()
   t4.start()



#######################
if __name__ == '__main__':
   n=Nextion()
   g=GPS()
   main()


'''
GPS Interfacing with Raspberry Pi using Pyhton
http://www.electronicwings.com
'''
import serial               #import serial pacakge
from time import sleep
import webbrowser           #import package for opening link in browser
import sys                  #import system package

def GPS_Info():
    global NMEA_buff
    global lat_in_degrees
    global long_in_degrees
    nmea_time = []
    local_time = []
    nmea_latitude = []
    nmea_longitude = []
    nmea_time = NMEA_buff[0]                    #extract time from GPGGA string
    local_time = NMEA_buff[0]
    nmea_latitude = NMEA_buff[1]                #extract latitude from GPGGA string
    nmea_longitude = NMEA_buff[3]               #extract longitude from GPGGA string
    hour  = nmea_time[0:2]
    min = nmea_time[2:4]
    sec = nmea_time[4:6]
    UTC_time = (hour + ":" + min + ":" +sec)
    vhour = int(hour)+13
    if vhour > 23: 
         vhour = vhour-24
    hour = str(vhour)

    local_time = (hour + ":" + min + ":" +sec)

    
    print ("UTC Time:   ", UTC_time)
    print ("Local Time: ", local_time)
    #print ("NMEA Latitude:", nmea_latitude,"NMEA Longitude:", nmea_longitude)
    
    lat = float(nmea_latitude)                  #convert string into float for calculation
    longi = float(nmea_longitude)               #convertr string into float for calculation
    
    lat_in_degrees = convert_to_degrees(lat)    #get latitude in degree decimal format
    long_in_degrees = convert_to_degrees(longi) #get longitude in degree decimal format
    
#convert raw NMEA string into degree decimal format   
def convert_to_degrees(raw_value):
    decimal_value = raw_value/100.00
    degrees = int(decimal_value)
    mm_mmmm = (decimal_value - int(decimal_value))/0.6
    position = degrees + mm_mmmm
    position = "%.4f" %(position)
    return position
    


gpgga_info = "$GPGGA,"
ser = serial.Serial ("/dev/ttyS0")              #Open port with baud rate
GPGGA_buffer = 0
NMEA_buff = 0
lat_in_degrees = 0
long_in_degrees = 0

try:
    while True:
        received_data = (str)(ser.readline())                   #read NMEA string received
        GPGGA_data_available = received_data.find(gpgga_info)   #check for NMEA GPGGA string                 
        if (GPGGA_data_available>0):
            GPGGA_buffer = received_data.split("$GPGGA,",1)[1]  #store data coming after "$GPGGA," string 
            NMEA_buff = (GPGGA_buffer.split(','))               #store comma separated data in buffer
            GPS_Info()                                          #get time, latitude, longitude
 
            print ("Latitude:", lat_in_degrees," Longitude: ", long_in_degrees )
            map_link = 'http://maps.google.com/?q=' + lat_in_degrees + ',' + long_in_degrees    #create link to plot location on Google map
            #print("<<<<<<<<press ctrl+c to plot location on google maps>>>>>>\n")               #press ctrl+c to plot on map and exit 
            print ("------------------------------------------------------------")
                        
except KeyboardInterrupt:
    webbrowser.open(map_link)        #open current position information in google map
    sys.exit(0)



import staticmaps
import numpy as np

# Latitude: 1 deg = 110.574 km
# Longitude: 1 deg = 111.320*cos(latitude) km

yourLat = 10  # Enter centre Latitude in degrees 
yourLon = -85  # Enter centre Longitude in degrees

screenx = 320.0
screeny = 240.0
zoomx = 9200       # x dimension in km (approx)
zoomy=zoomx*(screeny/screenx)
lonx = 111.32*np.cos(np.radians(yourLat))
lonxd = (zoomx/lonx)/2
laty = 110.574
latyd =(zoomy/laty)/2
xleft = round(yourLon-lonxd,4)
xright = round(yourLon+lonxd,4)
ytop = round(yourLat+latyd,4)
ybottom = round(yourLat-latyd,4)

if xleft<-180:
   xleft=-180
if xright>180:
   xright=180
if ytop>90:
   ytop=90
if ytop<-90:
   ytop=-90
if ybottom<-90:
   ybottom=-90
if ybottom>90:
   ybottom=90

print("calculating ...")
context = staticmaps.Context()
#context.set_tile_provider(staticmaps.tile_provider_StamenToner)
#context.set_tile_provider(staticmaps.tile_provider_ArcGISWorldImagery)
context.set_tile_provider(staticmaps.tile_provider_OSM)

polygon = [
    (ytop,xright),
    (ytop,xleft),
    (ybottom,xleft),  
    (ybottom,xright),
    (ytop,xright),
]
print(yourLat,yourLon)
print(ytop,xleft)
print(ybottom,xright)

zoom = 2  # radius in km 
context.add_object(staticmaps.Area([staticmaps.create_latlng(lat, lng) for lat, lng in polygon],
        fill_color=staticmaps.TRANSPARENT,width=2,color=staticmaps.parse_color("#00FF003F")))

#context.add_object(staticmaps.Circle(center1, zoom, fill_color=staticmaps.TRANSPARENT, color=staticmaps.RED, width=2))
#context.add_object(staticmaps.Marker(center1, color=staticmaps.RED, size=6))

# render non-anti-aliased png
image = context.render_pillow(640, 480)
image.save("map.png")
print("saved the map as map.png")



import setuptools

with open("README.md", "r") as fh:

    long_description = fh.read()

setuptools.setup(
    name="GPS_clock",
    version="0.1.0",
    url="https://www.qsl.net/zl3dw/",
    author="Andrew Barron",
    author_email="zl3dw@outlook.co.nz",
    description="A GPS clock and Nextion display tft writer",
    long_description=long_description,
    packages=setuptools.find_packages(),
    install_requires=[],
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
    ],
    include_package_data=True,
    package_data={'': ['ref/*.*']},
)